from enum import Enum
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict


class CaseInsensitiveEnum(str, Enum):
    @classmethod
    def _missing_(cls, value: Any) -> Optional[str]:
        if isinstance(value, str):
            value = value.lower()
            for member in cls:
                if member.lower() == value:
                    return member
        return None


class Aggregation(str, Enum):
    HOUR = "hour"
    DAY = "day"
    WEEK = "week"
    MONTH = "month"
    QUARTER = "quarter"
    YEAR = "year"
    MILLENNIUM = "millennium"


class BaseConfigModel(BaseModel):
    model_config = ConfigDict(extra="forbid")



from enum import IntEnum
from typing import Annotated, Any, Optional, Union

from pydantic import BaseModel, BeforeValidator, Field, SerializerFunctionWrapHandler, WrapSerializer


class OperationType(IntEnum):
    """
    XXX: Hey developer! This enum has to match the fields of `message Operation` below
    """

    graph = 0
    label = 1
    ask = 2
    qa = 3
    extract = 4
    prompt_guard = 5
    llama_guard = 6


# Custom serialization so clients get the string representation of the OperationType enum instead of the int
# This has to be done because the IntEnum is forced by protobuf definition
def serialize_operation_type(value: OperationType, nxt: SerializerFunctionWrapHandler) -> str:
    return value.name


# To accept both string and int values for OperationType
def validate_operation_type(value: Union[str, int]) -> Union[OperationType, int]:
    if isinstance(value, str):
        try:
            return OperationType[value]
        except KeyError as e:
            msg = f"Invalid OperationType {value}, must be one of {OperationType._member_names_}"
            raise ValueError(msg) from e
    return value


OperationTypeString = Annotated[
    OperationType,
    WrapSerializer(serialize_operation_type),
    BeforeValidator(validate_operation_type),
]


class NameOperationFilter(BaseModel):
    """
    Filtering Data Augmentation Agents by operation type and task names.
    """

    operation_type: OperationTypeString = Field(..., description="Type of the operation")
    task_names: list[str] = Field(
        default_factory=list,
        description="List of task names. If None or empty, all tasks for that operation are applied.",
    )


class Ask(BaseModel):
    text: Optional[str] = None
    json_output: Optional[dict[str, Any]] = None
    empty: bool = False


class Position(BaseModel):
    start: int
    end: int


class Entities(BaseModel):
    labels: dict[str, str]
    positions: dict[str, list[Position]]


class RelationNode(BaseModel):
    """
    Represents a relation between two entities.

    """

    entity: str
    label: str
    position: Position


# XXX: We have this model here instead of using existing Relation models
# because this one is sent with triggers
class Relation(BaseModel):
    """
    Represents a relation between two entities.

    """

    head: RelationNode
    tail: RelationNode
    label: str
    paragraph_id: str


class Relations(BaseModel):
    relations: list[Relation]


class Labels(BaseModel):
    labels: dict[str, Union[str, list[str]]]


class Question(BaseModel):
    question: str
    answer: str
    block: int
    reasoning: str
    paragraph_id: str


class Payload(BaseModel):
    type: OperationTypeString = Field(
        ...,
        description=f"The type of operation performed, can be any of {OperationType._member_names_}",
    )
    task_name: str = Field(..., description="The name of the task that generated this payload")
    errors: list[str] = Field(
        default_factory=list,
        description="List of errors (if any) that occurred while processing this payload",
    )
    asks: Optional[list[Ask]] = Field(default=None, description="Results of the ask operation")
    relations: Optional[list[Relations]] = Field(
        default=None, description="Relation results for the graph operation"
    )
    entities: Optional[list[Entities]] = Field(
        default=None, description="Entity results for the graph operation"
    )
    labels: Optional[list[Labels]] = Field(
        default=None, description="Label results for the labeler operation"
    )
    guard_labels: Optional[list[str]] = Field(
        default=None, description="Guard labels for the llama_guard operation"
    )
    prompt_guard: Optional[list[str]] = Field(
        default=None, description="Prompt guard labels for the prompt_guard operation"
    )
    qas: Optional[list[Question]] = Field(
        default=None,
        description="Question and answer results for the synthetic_questions operation",
    )
    kbid: str
    field: str
    rid: Optional[str] = None



from typing import Literal

from pydantic import BaseModel


class TokensDetail(BaseModel):
    input: float
    output: float
    image: float


class Consumption(BaseModel):
    normalized_tokens: TokensDetail
    customer_key_tokens: TokensDetail


class ConsumptionGenerative(Consumption):
    type: Literal["consumption"] = "consumption"



from typing import no_type_check

import pytest

from nuclia_models.events.activity_logs import (
    DEFAULT_SHOW_ASK_VALUES,
    DEFAULT_SHOW_CHAT_VALUES,
    DEFAULT_SHOW_SEARCH_VALUES,
    DEFAULT_SHOW_VALUES,
    ActivityLogsAsk,
    ActivityLogsAskQuery,
    ActivityLogsChat,
    ActivityLogsChatQuery,
    DownloadActivityLogsAskQuery,
    DownloadActivityLogsChatQuery,
    QueryFiltersAsk,
    QueryFiltersChat,
    QueryFiltersCommon,
    QueryFiltersSearch,
    QuestionFilter,
)


def test_query_filters_fields() -> None:
    chat_fields = set(QueryFiltersChat.model_fields.keys())
    ask_fields = set(QueryFiltersAsk.model_fields.keys())
    search_fields = set(QueryFiltersSearch.model_fields.keys())
    common_fields = set(QueryFiltersCommon.model_fields.keys())
    question_fields = set(QuestionFilter.model_fields.keys())

    # common_fields are part of chat, ask and search fields
    assert common_fields.issubset(chat_fields)
    assert common_fields.issubset(ask_fields)
    assert common_fields.issubset(search_fields)

    # question_fields are part of chat and ask
    assert question_fields.issubset(chat_fields)
    assert question_fields.issubset(ask_fields)

    # chat_fields + search_fields = ask_fields
    assert chat_fields.union(search_fields) == ask_fields

    # any of the search fields are part of the chat_fields (except the common fields and question fields)
    non_common_search_fields = search_fields - common_fields
    assert non_common_search_fields.intersection(chat_fields - common_fields) == question_fields


@no_type_check
def test_ask_and_chat_does_not_warn(recwarn: pytest.WarningsRecorder) -> None:
    _ = QueryFiltersAsk()
    _ = DownloadActivityLogsAskQuery(year_month="2025-05", filters=QueryFiltersAsk())
    _ = ActivityLogsAsk(year_month="2025-05", filters=QueryFiltersAsk())
    _ = ActivityLogsAskQuery(year_month="2025-05", filters=QueryFiltersAsk())

    _ = QueryFiltersChat()
    _ = DownloadActivityLogsChatQuery(year_month="2025-05", filters=QueryFiltersChat())
    _ = ActivityLogsChat(year_month="2025-05", filters=QueryFiltersChat())
    _ = ActivityLogsChatQuery(year_month="2025-05", filters=QueryFiltersChat())

    warnings_ = recwarn.list
    assert len(warnings_) == 0, "Expected no warnings"


def test_default_show_values() -> None:
    assert {"id", "date"} == DEFAULT_SHOW_VALUES
    assert {"id", "date", "question", "resources_count"} == DEFAULT_SHOW_SEARCH_VALUES
    assert {
        "id",
        "date",
        "question",
        "rephrased_question",
        "rag_strategies_names",
        "answer",
    } == DEFAULT_SHOW_CHAT_VALUES
    assert {
        "id",
        "date",
        "question",
        "rephrased_question",
        "rag_strategies_names",
        "answer",
        "resources_count",
    } == DEFAULT_SHOW_ASK_VALUES



from typing import Optional

from pydantic import BaseModel, Field, model_validator
from typing_extensions import Self

from nuclia_models.common.consumption import Consumption


class RemiRequest(BaseModel):
    """
    Model to represent a request for the REMi model

    Metrics will be computed accordingly to the inputs given

    - Answer relevance will be computed if `answer` and `question` are provided
    - Context relevance will be computed if `question` and `contexts` are provided
    - Groundedness will be computed if `answer` and `contexts` are provided
    """

    user_id: str = Field(..., title="The user ID of the user making the request")
    question: Optional[str] = Field(default=None, title="The question or query that the user asked")
    answer: Optional[str] = Field(default=None, title="The answer that the model provided")
    contexts: Optional[list[str]] = Field(
        default=None, title="The contexts that the model used to generate the answer"
    )

    @model_validator(mode="after")
    def check_metrics(self) -> Self:
        if not any([self.question, self.answer, self.contexts]):
            message = "At least one of question, answer or contexts must be provided"
            raise ValueError(message)
        return self


class AnswerRelevance(BaseModel):
    score: int = Field(..., title="The score of the answer relevance")
    reason: str = Field(..., title="The reasoning for the score")


class RemiResponse(BaseModel):
    time: float = Field(..., title="The time taken to compute the response")
    answer_relevance: Optional[AnswerRelevance] = Field(
        default=None,
        title="The relevance score of the answer to the question.",
        description="""Answer Relevance measures the relevance of the generated answer to the user query, \
            in a scale of 0 to 5.""",
    )
    context_relevance: Optional[list[Optional[int]]] = Field(
        default=None,
        title="The relevance score of each context to the question.",
        description="""Context Relevance measures the relevance of the retrieved context to the user query, \
        in a scale of 0 to 5. The score will be None if there was an error computing the score for a specific\
         context.""",
    )
    groundedness: Optional[list[Optional[int]]] = Field(
        default=None,
        title="The groundedness score of the answer on each context.",
        description="""Groundedness measures the degree to which the generated answer is grounded in the \
        retrieved context, in a scale of 0 to 5. \
        The score will be None if there was an error computing the score for a specific context.""",
    )
    consumption: Optional[Consumption] = Field(
        default=None,
        title="The consumption details for the REMi model run",
    )



from typing import Annotated, Optional

from pydantic import AfterValidator, BaseModel, Field

from nuclia_models.worker.triggers import NameOperationFilter, Payload


def validate_filters(
    filters: Optional[list[NameOperationFilter]],
) -> Optional[list[NameOperationFilter]]:
    if filters is None:
        return []
    # Check that the filters are unique
    seen = set()
    for filter in filters:
        if filter.operation_type in seen:
            msg = f"Duplicate operation type for type `{filter.operation_type.name}` in filters"
            raise ValueError(msg)
        seen.add(filter.operation_type)
    return filters


FiltersType = Annotated[Optional[list[NameOperationFilter]], AfterValidator(validate_filters)]


class RunAgentsRequestBase(BaseModel):
    user_id: str = Field(..., title="The user ID of the user making the request")
    filters: FiltersType = Field(
        default_factory=list,  # type: ignore
        title="Filters to select which Data Augmentation Agents are applied to the text. If empty, \
        all configured agents for the Knowledge Box are applied.",
    )
    enable_webhooks: bool = Field(
        default=False,
        title="Whether to enable the triggering of the configured webhooks for each applied \
        Data Augmentation Agent",
    )


class RunTextAgentsRequest(RunAgentsRequestBase):
    texts: list[str] = Field(
        ...,
        title="The texts to run the agents on",
    )


class AugmentedBase(BaseModel):
    input_nuclia_tokens: float = Field(..., title="The number of input Nuclia tokens consumed for the field")
    output_nuclia_tokens: float = Field(
        ..., title="The number of output Nuclia tokens consumed for the field"
    )
    time: float = Field(..., title="The time taken to execute the Data Augmentation agents to the field")


class AugmentedText(AugmentedBase):
    payloads: list[Payload] = Field(
        ...,
        title="The list of payloads generated by the Data Augmentation agents for a given text",
    )


class RunTextAgentsResponse(BaseModel):
    results: list[AugmentedText] = Field(
        ...,
        title="Payloads generated by the agents for each text",
    )



from datetime import datetime
from enum import Enum
from typing import Literal, Optional

from pydantic import BaseModel

from nuclia_models.common.pagination import Pagination


class ContextRelevanceQuery(BaseModel):
    value: int
    operation: Literal["gt", "lt", "eq"]
    aggregation: Literal["average", "max", "min"]


class Status(Enum):
    SUCCESS = "SUCCESS"
    ERROR = "ERROR"
    NO_CONTEXT = "NO_CONTEXT"

    def get_value(self) -> int:
        mapping = {
            Status.SUCCESS: 0,
            Status.ERROR: -1,
            Status.NO_CONTEXT: -2,
        }
        return mapping[self]


class RemiQuery(BaseModel):
    context_relevance: Optional[ContextRelevanceQuery] = None
    month: str
    feedback_good: Optional[bool] = None
    status: Optional[Status] = None
    pagination: Pagination = Pagination()


class RetrievedContext(BaseModel):
    text: str
    text_block_id: Optional[str] = None


class RemiAnswerRelevance(BaseModel):
    score: int
    reason: str


class RemiScores(BaseModel):
    answer_relevance: RemiAnswerRelevance
    context_relevance: list[int]
    groundedness: list[int]


class RemiQueryResult(BaseModel):
    id: int
    question: str
    answer: Optional[str]
    remi: Optional[RemiScores]


class RemiQueryResultWithContext(RemiQueryResult):
    context: list[RetrievedContext]


class RemiQueryResults(BaseModel):
    data: list[RemiQueryResult]
    has_more: bool


class AggregatedRemiScoreValues(BaseModel):
    name: str
    min: float
    max: float
    average: float


class AggregatedRemiScoreMetric(BaseModel):
    timestamp: datetime
    metrics: list[AggregatedRemiScoreValues]



import re
from datetime import datetime
from enum import Enum
from typing import Annotated, Any, Generic, Literal, Optional, TypeVar, Union

from pydantic import BaseModel, ConfigDict, EmailStr, Field, field_validator, model_validator
from pydantic.main import create_model

from nuclia_models.common.client import ClientType
from nuclia_models.common.pagination import Pagination
from nuclia_models.common.user import UserType
from nuclia_models.common.utils import CaseInsensitiveEnum

T = TypeVar("T")


class EventType(CaseInsensitiveEnum):
    # Nucliadb
    VISITED = "visited"
    MODIFIED = "modified"
    DELETED = "deleted"
    NEW = "new"
    SEARCH = "search"
    SUGGEST = "suggest"
    INDEXED = "indexed"
    CHAT = "chat"
    ASK = "ask"
    # Tasks
    STARTED = "started"
    STOPPED = "stopped"
    # Processor
    PROCESSED = "processed"


class BaseConfigModel(BaseModel):
    model_config = ConfigDict(extra="forbid")


class ShowOnly(BaseConfigModel):
    """
    Marker class to indicate that a field is for display purposes only.

    Fields wrapped in this class are excluded from OpenAPI documentation to prevent confusion,
    as they are not intended for filtering. Instead, they are only used in the "show" parameter.

    This behavior is enforced in the FastAPI application through introspection,
    ensuring these fields are removed from the generated API documentation.
    """


class GenericFilter(BaseConfigModel, Generic[T]):
    eq: Optional[T] = None
    gt: Optional[T] = None
    ge: Optional[T] = None
    lt: Optional[T] = None
    le: Optional[T] = None
    ne: Optional[T] = None
    isnull: Optional[bool] = None


class StringFilter(GenericFilter[str]):
    like: Optional[str] = None
    ilike: Optional[str] = None


class AuditMetadata(StringFilter):
    key: str


class QueryFiltersCommon(BaseConfigModel):
    id: Optional[GenericFilter[int]] = None
    date: Optional[ShowOnly] = Field(None, serialization_alias="event_date")
    user_id: Optional[GenericFilter[str]] = None
    user_type: Optional[GenericFilter[UserType]] = None
    client_type: Optional[GenericFilter[ClientType]] = None
    total_duration: Optional[GenericFilter[int]] = None
    audit_metadata: Optional[list[AuditMetadata]] = Field(
        None, serialization_alias="data.user_request.audit_metadata"
    )
    resource_id: Optional[ShowOnly] = None
    nuclia_tokens: Optional[GenericFilter[float]] = Field(
        None, serialization_alias="nuclia_tokens.billable_nuclia_tokens"
    )
    token_details: Optional[ShowOnly] = Field(None, serialization_alias="nuclia_tokens.token_details")


class QuestionFilter(BaseModel):
    question: Optional[StringFilter] = Field(None, serialization_alias="data.user_request.query")


class QueryFiltersSearch(QueryFiltersCommon, QuestionFilter):
    resources_count: Optional[GenericFilter[int]] = Field(
        None,
        serialization_alias="data.resources_count",
        # cast_to is only needed for JSONB columns, it allows values: str, bool, int, float
        json_schema_extra={"cast_to": "int"},
    )
    filter: Optional[ShowOnly] = Field(None, serialization_alias="data.request.filter")
    retrieval_rephrased_question: Optional[ShowOnly] = Field(
        None, serialization_alias="data.request.retrieval_rephrased_question"
    )
    vectorset: Optional[StringFilter] = Field(None, serialization_alias="data.request.vectorset")
    security: Optional[ShowOnly] = Field(None, serialization_alias="data.request.security")
    min_score_bm25: Optional[GenericFilter[float]] = Field(
        None, serialization_alias="data.request.min_score_bm25", json_schema_extra={"cast_to": "float"}
    )
    min_score_semantic: Optional[GenericFilter[float]] = Field(
        None, serialization_alias="data.request.min_score_semantic", json_schema_extra={"cast_to": "float"}
    )
    result_per_page: Optional[GenericFilter[int]] = Field(
        None, serialization_alias="data.request.result_per_page", json_schema_extra={"cast_to": "int"}
    )
    retrieval_time: Optional[GenericFilter[int]] = Field(
        None,
        serialization_alias="data.retrieval_time",
        json_schema_extra={"cast_to": "int"},
        description=(
            "Time in milliseconds spent on the NucliaDB side for retrieval, including retrieval rephrase,"
            " calculation of query embedding, index search and results hydration"
        ),
    )


class QueryFiltersChat(QueryFiltersCommon, QuestionFilter):
    rephrased_question: Optional[StringFilter] = Field(
        None, serialization_alias="data.request.rephrased_question"
    )
    answer: Optional[StringFilter] = Field(None, serialization_alias="data.request.answer")
    learning_id: Optional[ShowOnly] = Field(None, serialization_alias="data.request.learning_id")
    retrieved_context: Optional[ShowOnly] = Field(None, serialization_alias="data.request.context")
    chat_history: Optional[ShowOnly] = Field(None, serialization_alias="data.request.chat_context")
    feedback_good: Optional[GenericFilter[bool]] = Field(
        None,
        serialization_alias="data.feedback.good",
        json_schema_extra={"cast_to": "bool"},
        description="True if the feedback provided for the main question is positive.",
    )
    feedback_comment: Optional[StringFilter] = Field(
        None,
        serialization_alias="data.feedback.feedback",
        description="User-provided comment on the feedback for the question.",
    )
    feedback_good_all: Optional[GenericFilter[bool]] = Field(
        None,
        serialization_alias="data.feedback.all",
        json_schema_extra={"cast_to": "bool"},
        description=(
            "True if all feedback, including that on the main question"
            " and each related text block, is positive."
        ),
    )
    feedback_good_any: Optional[GenericFilter[bool]] = Field(
        None,
        serialization_alias="data.feedback.any",
        json_schema_extra={"cast_to": "bool"},
        description=(
            "True if there is any positive feedback" " on the question itself or any related text block."
        ),
    )
    feedback: Optional[ShowOnly] = Field(
        None,
        serialization_alias="data.feedback",
        description=(
            "Raw feedback data associated with the question or generative answer,"
            " including feedback on related text blocks."
        ),
    )
    model: Optional[StringFilter] = Field(None, serialization_alias="data.request.model")
    rag_strategies_names: Optional[ShowOnly] = Field(None, serialization_alias="data.rag_strategies")
    rag_strategies: Optional[ShowOnly] = Field(None, serialization_alias="data.user_request.rag_strategies")
    status: Optional[GenericFilter[int]] = Field(
        None,
        serialization_alias="data.request.status_code",
        json_schema_extra={"cast_to": "int"},
    )
    generative_answer_first_chunk_time: Optional[GenericFilter[int]] = Field(
        None,
        serialization_alias="data.generative_answer_first_chunk_time",
        json_schema_extra={"cast_to": "int"},
        description=(
            "Time in milliseconds from when the user made the request to when the first answer"
            " chunk of data was returned by the generative model."
        ),
    )
    generative_reasoning_first_chunk_time: Optional[GenericFilter[int]] = Field(
        None,
        serialization_alias="data.generative_reasoning_first_chunk_time",
        json_schema_extra={"cast_to": "int"},
        description=(
            "Time in milliseconds from when the user made the request to when the first reasoning"
            " chunk of data was returned by the generative model."
        ),
    )
    generative_answer_time: Optional[GenericFilter[int]] = Field(
        None,
        serialization_alias="data.generative_answer_time",
        json_schema_extra={"cast_to": "int"},
        description=(
            "Time in milliseconds elapsed between the answer streaming request is done"
            " to the generative model until the last chunk of the answer is returned."
        ),
    )
    remi_scores: Optional[ShowOnly] = Field(None, serialization_alias="data_eval")
    user_request: Optional[ShowOnly] = Field(None, serialization_alias="data.user_request")
    reasoning: Optional[StringFilter] = Field(None, serialization_alias="data.request.reasoning")


class QueryFiltersAsk(QueryFiltersSearch, QueryFiltersChat):
    pass


def create_dynamic_model(name: str, base_model: type[QueryFiltersAsk]) -> type[BaseModel]:
    field_definitions = {}
    # Any field that is not str should be added here, this is used for the API activity log schema response
    field_type_map = {
        "id": int,
        "user_type": Optional[UserType],
        "client_type": Optional[ClientType],
        "total_duration": Optional[float],
        "generative_answer_first_chunk_time": Optional[int],
        "generative_answer_time": Optional[int],
        "resources_count": Optional[int],
        "feedback_good": Optional[bool],
        "feedback_good_all": Optional[bool],
        "feedback_good_any": Optional[bool],
        "feedback": Optional[dict],
        "status": Optional[int],
        "rag_strategies": Optional[list],
        "rag_strategies_names": Optional[list],
        "chat_history": Optional[list],
        "retrieved_context": Optional[list],
        "nuclia_tokens": Optional[float],
        "remi_scores": Optional[dict],
        "token_details": Optional[dict],
        "user_request": Optional[dict],
        "min_score_bm25": Optional[float],
        "min_score_semantic": Optional[float],
        "result_per_page": Optional[int],
        "retrieval_time": Optional[int],
    }
    for field_name in base_model.model_fields:
        field_type = field_type_map.get(field_name, Optional[str])

        field_definitions[field_name] = (field_type, Field(default=None))

    return create_model(name, **field_definitions)  # type: ignore


ActivityLogsQueryResponse = create_dynamic_model(name="ActivityLogsQueryResponse", base_model=QueryFiltersAsk)


class ActivityLogsQueryCommon(BaseConfigModel):
    year_month: str

    @field_validator("year_month")
    @classmethod
    def validate_year_month(cls, value: str) -> str:
        if not re.match(r"^\d{4}-(0[1-9]|1[0-2])$", value):
            msg = "year_month must be in the format YYYY-MM"
            raise ValueError(msg)
        return value

    @staticmethod
    def _validate_show(
        show: Union[set[str], Literal["all"]], model: type[QueryFiltersCommon]
    ) -> Union[set[str], Literal["all"]]:
        if show == "all":
            return show

        allowed_fields = list(model.model_fields.keys())
        for field in show:
            if field.startswith("audit_metadata."):
                continue
            if field not in allowed_fields:
                msg = f"{field} is not a field. List of fields: {allowed_fields}"
                raise ValueError(msg)
        return show


SHOW_LITERAL = Literal[tuple(QueryFiltersCommon.model_fields.keys())]  # type: ignore
SHOW_SEARCH_LITERAL = Literal[tuple(QueryFiltersSearch.model_fields.keys())]  # type: ignore
SHOW_CHAT_LITERAL = Literal[tuple(QueryFiltersChat.model_fields.keys())]  # type: ignore
SHOW_ASK_LITERAL = Literal[tuple(QueryFiltersAsk.model_fields.keys())]  # type: ignore
DEFAULT_SHOW_VALUES = {"id", "date"}
DEFAULT_SHOW_SEARCH_VALUES = DEFAULT_SHOW_VALUES | {"question", "resources_count"}
DEFAULT_SHOW_CHAT_VALUES = {
    "question",
    "rephrased_question",
    "answer",
    "rag_strategies_names",
} | DEFAULT_SHOW_VALUES
DEFAULT_SHOW_ASK_VALUES = DEFAULT_SHOW_SEARCH_VALUES | DEFAULT_SHOW_CHAT_VALUES


class ActivityLogs(ActivityLogsQueryCommon):
    show: Union[set[SHOW_LITERAL], Literal["all"]] = DEFAULT_SHOW_VALUES  # type: ignore
    filters: QueryFiltersCommon

    @field_validator("show")
    @classmethod
    def validate_show(cls, show: Union[set[str], Literal["all"]]) -> Union[set[str], Literal["all"]]:
        return cls._validate_show(show=show, model=QueryFiltersCommon)


class ActivityLogsSearch(ActivityLogsQueryCommon):
    show: Union[set[SHOW_SEARCH_LITERAL], Literal["all"]] = DEFAULT_SHOW_SEARCH_VALUES  # type: ignore
    filters: QueryFiltersSearch

    @field_validator("show")
    @classmethod
    def validate_show(cls, show: Union[set[str], Literal["all"]]) -> Union[set[str], Literal["all"]]:
        return cls._validate_show(show=show, model=QueryFiltersSearch)


class ActivityLogsChat(ActivityLogsQueryCommon):
    show: Union[set[SHOW_CHAT_LITERAL], Literal["all"]] = DEFAULT_SHOW_CHAT_VALUES  # type: ignore
    filters: QueryFiltersChat

    @field_validator("show")
    @classmethod
    def validate_show(cls, show: Union[set[str], Literal["all"]]) -> Union[set[str], Literal["all"]]:
        return cls._validate_show(show=show, model=QueryFiltersChat)


class ActivityLogsAsk(ActivityLogsQueryCommon):
    show: Union[set[SHOW_ASK_LITERAL], Literal["all"]] = DEFAULT_SHOW_ASK_VALUES  # type: ignore
    filters: QueryFiltersAsk

    @field_validator("show")
    @classmethod
    def validate_show(cls, show: Union[set[str], Literal["all"]]) -> Union[set[str], Literal["all"]]:
        return cls._validate_show(show=show, model=QueryFiltersAsk)


class PaginationMixin(BaseModel):
    pagination: Pagination = Pagination()

    @model_validator(mode="after")
    @classmethod
    def validate_pagination_and_filters(cls, values):  # type: ignore
        if values.pagination and values.filters and values.filters.id is not None:
            raise ValueError("Payload cannot have both 'pagination' and an 'id' in 'filters'.")  # noqa: TRY003, EM101
        return values


class ActivityLogsSearchQuery(ActivityLogsSearch, PaginationMixin):
    pass


class ActivityLogsChatQuery(ActivityLogsChat, PaginationMixin):
    pass


class ActivityLogsAskQuery(ActivityLogsAsk, PaginationMixin):
    pass


class ActivityLogsQuery(ActivityLogs, PaginationMixin):
    pass


class DownloadRequestType(str, Enum):
    QUERY = "query"


class DownloadFormat(str, Enum):
    NDJSON = "ndjson"
    CSV = "csv"


class DownloadRequest(BaseModel):
    id: Annotated[int, Field(exclude=True)]
    request_id: str
    download_type: DownloadRequestType
    download_format: DownloadFormat
    event_type: EventType
    requested_at: datetime
    user_id: Annotated[str, Field(exclude=True)]
    kb_id: str
    query: Annotated[dict[Any, Any], Field(exclude=True)]
    download_url: Optional[str]

    # Configuration for Pydantic v2 to handle ORM mapping
    class Config:
        from_attributes = True


class DownloadActivityLogsQueryMixin(BaseModel):
    email_address: Optional[EmailStr] = Field(default=None)
    notify_via_email: bool = Field(default=False)


class DownloadActivityLogsSearchQuery(DownloadActivityLogsQueryMixin, ActivityLogsSearch):
    pass


class DownloadActivityLogsChatQuery(DownloadActivityLogsQueryMixin, ActivityLogsChat):
    pass


class DownloadActivityLogsAskQuery(DownloadActivityLogsQueryMixin, ActivityLogsAsk):
    pass


class DownloadActivityLogsQuery(DownloadActivityLogsQueryMixin, ActivityLogs):
    pass



from nuclia_models.worker.proto import AskOperation


def test_ask_operation_alias() -> None:
    # Can be constructed using "json"
    op = AskOperation(question="q", destination="d", json=True)
    assert op.generate_json is True

    # Can be deserialized from "json"
    op = AskOperation.model_validate_json("""{"question": "q", "destination": "d", "json" : true}""")
    assert op.generate_json is True

    # Serializez to "json"
    op = AskOperation(question="q", destination="d")
    op.generate_json = True
    assert (
        op.model_dump_json()
        == """{"question":"q","destination":"d","json":true,"triggers":[],"user_prompt":""}"""
    )



from typing import Any, Literal, Optional, Union

from pydantic import BaseModel, Field

from nuclia_models.common.consumption import Consumption, ConsumptionGenerative

GenerativeResponseType = Literal["text", "object", "meta", "citations", "status", "footnote_citations"]


class TextGenerativeResponse(BaseModel):
    type: Literal["text"] = "text"
    text: str


class JSONGenerativeResponse(BaseModel):
    type: Literal["object"] = "object"
    object: dict[str, Any]


class MetaGenerativeResponse(BaseModel):
    type: Literal["meta"] = "meta"
    input_tokens: int
    output_tokens: int
    timings: dict[str, float]
    input_nuclia_tokens: Optional[float] = None
    output_nuclia_tokens: Optional[float] = None


class CitationsGenerativeResponse(BaseModel):
    type: Literal["citations"] = "citations"
    citations: dict[str, Any]


class FootnoteCitationsGenerativeResponse(BaseModel):
    """Maps ids in the footnote citations to query_context keys (normally paragraph ids)
    e.g.,
    { "block-AA": "f44f4e8acbfb1d48de3fd3c2fb04a885/f/f44f4e8acbfb1d48de3fd3c2fb04a885/73758-73972", ... }
    If the query_context is a list, it will map to 1-based indices as strings
    e.g., { "block-AA": "1", "block-AB": "2", ... }
    """

    type: Literal["footnote_citations"] = "footnote_citations"
    footnote_to_context: dict[str, str]


class RerankGenerativeResponse(BaseModel):
    type: Literal["rerank"] = "rerank"
    context_scores: dict[str, float]


class StatusGenerativeResponse(BaseModel):
    type: Literal["status"] = "status"
    code: str
    details: Optional[str] = None


class CallArguments(BaseModel):
    name: Optional[str]
    arguments: dict[str, Any]


class ToolCall(BaseModel):
    function: CallArguments
    id: Optional[str] = None


class ToolsGenerativeResponse(BaseModel):
    type: Literal["tools"] = "tools"
    tools: dict[str, list[ToolCall]]


class ReasoningGenerativeResponse(BaseModel):
    type: Literal["reasoning"] = "reasoning"
    text: str


GenerativeResponse = Union[
    TextGenerativeResponse,
    ReasoningGenerativeResponse,
    JSONGenerativeResponse,
    MetaGenerativeResponse,
    CitationsGenerativeResponse,
    FootnoteCitationsGenerativeResponse,
    StatusGenerativeResponse,
    RerankGenerativeResponse,
    ToolsGenerativeResponse,
    ConsumptionGenerative,
]


class GenerativeChunk(BaseModel):
    chunk: GenerativeResponse = Field(..., discriminator="type")


class GenerativeFullResponse(BaseModel):
    input_tokens: Optional[int] = None  # TODO: deprecate
    output_tokens: Optional[int] = None  # TODO: deprecate
    timings: Optional[dict[str, float]] = None
    citations: Optional[dict[str, Any]] = None
    citation_footnote_to_context: Optional[dict[str, str]] = Field(
        default=None,
        description="For the LLM footnote citations feature, "
        "maps footnote ids to context keys (normally paragraph ids)",
    )
    code: Optional[str] = None
    details: Optional[str] = None
    answer: str
    reasoning: Optional[str] = None
    object: Optional[dict[str, Any]] = None
    input_nuclia_tokens: Optional[float] = None  # TODO: deprecate
    output_nuclia_tokens: Optional[float] = None  # TODO: deprecate
    tools: Optional[dict[str, list[ToolCall]]] = None
    consumption: Optional[Consumption] = None



# ruff: noqa: UP006
# ruff: noqa: E501
import typing
from datetime import datetime
from enum import IntEnum

from pydantic import BaseModel, ConfigDict, Field

# The models from this file are generated from private protos
# Here we just duplicate the models


###
#    learning_protos/config_p2p.py
###


class SimilarityFunction(IntEnum):
    """
    Keep this in sync with learning_models.encoder.SemanticConfig
    """

    DOT = 0
    COSINE = 1


class ResponseStatus(IntEnum):
    OK = 0
    ERROR = 1
    VALIDATION_ERROR = 2
    NOT_FOUND = 3


class ModelType(IntEnum):
    """
    ALERT: needs to be synched to learning_config/models/ModelTypes
    """

    GENERATIVE = 0
    NER = 1
    RESOURCE_LABELER = 2
    CLASSIFIER = 3
    ANONYMIZER = 4
    VISUAL_LABELER = 5
    SUMMARY = 6
    DUMMY = 7
    PARAGRAPH_LABELER = 8
    EMBEDDINGS = 9
    RELATIONS = 10


class OpenAIKey(BaseModel):
    key: str = Field(default="")
    org: str = Field(default="")


class AzureOpenAIKey(BaseModel):
    key: str = Field(default="")
    url: str = Field(default="")
    deployment: str = Field(default="")
    model: str = Field(default="")


class HFLLMKey(BaseModel):
    class ModelType(IntEnum):
        LLAMA31 = 0
        QWEN25 = 1

    key: str = Field(default="")
    url: str = Field(default="")
    model: "HFLLMKey.ModelType" = Field(default=ModelType.LLAMA31)


class AzureMistralKey(BaseModel):
    key: str = Field(default="")
    url: str = Field(default="")


class PalmKey(BaseModel):
    credentials: str = Field(default="")
    location: str = Field(default="")


class MistralKey(BaseModel):
    key: str = Field(default="")


class AnthropicKey(BaseModel):
    key: str = Field(default="")


class TextGenerationKey(BaseModel):
    model: str = Field(default="")


class HFEmbeddingKey(BaseModel):
    """
         Some models require a specific template (including prefix) to work correctly in each task
    For example Snowflake's Arctic-embed requires a specific prefix to work correctly.
    In that case, the query prompt will be
    ```
    passage_prompt: ""
    query_prompt: "Represent this sentence for searching relevant passages: {}"
    ````
    where {} will be replaced by the actual sentence.
    `passage_prompt` is empty because the model does not require alterations to the sentence to embed is as a passage.
    """

    url: str = Field(default="")
    key: str = Field(default="")
    matryoshka: typing.List[int] = Field(default_factory=list)
    similarity: str = Field(default="")
    size: int = Field(default=0)
    threshold: float = Field(default=0.0)
    passage_prompt: str = Field(default="")
    query_prompt: str = Field(default="")


class UserLearningKeys(BaseModel):
    openai: typing.Optional[OpenAIKey] = Field(default=None)
    azure_openai: typing.Optional[AzureOpenAIKey] = Field(default=None)
    palm: typing.Optional[PalmKey] = Field(default=None)
    anthropic: typing.Optional[AnthropicKey] = Field(default=None)
    claude3: typing.Optional[AnthropicKey] = Field(default=None)
    text_generation: typing.Optional[TextGenerationKey] = Field(default=None)
    mistral: typing.Optional[MistralKey] = Field(default=None)
    azure_mistral: typing.Optional[AzureMistralKey] = Field(default=None)
    hf_llm: typing.Optional[HFLLMKey] = Field(default=None)
    hf_embedding: typing.Optional[HFEmbeddingKey] = Field(default=None)


class OpenAIUserPrompt(BaseModel):
    system: str = Field(default="")
    prompt: str = Field(default="")


class AzureUserPrompt(BaseModel):
    system: str = Field(default="")
    prompt: str = Field(default="")


class HFUserPrompt(BaseModel):
    system: str = Field(default="")
    prompt: str = Field(default="")


class PalmUserPrompt(BaseModel):
    prompt: str = Field(default="")


class AnthropicUserPrompt(BaseModel):
    prompt: str = Field(default="")


class Claude3UserPrompt(BaseModel):
    system: str = Field(default="")
    prompt: str = Field(default="")


class TextGenerationUserPrompt(BaseModel):
    prompt: str = Field(default="")


class MistralUserPrompt(BaseModel):
    prompt: str = Field(default="")


class AzureMistralUserPrompt(BaseModel):
    prompt: str = Field(default="")


class SummaryPrompt(BaseModel):
    prompt: str = Field(default="")


class UserPrompts(BaseModel):
    openai: typing.Optional[OpenAIUserPrompt] = Field(default=None)
    azure_openai: typing.Optional[AzureUserPrompt] = Field(default=None)
    palm: typing.Optional[PalmUserPrompt] = Field(default=None)
    anthropic: typing.Optional[AnthropicUserPrompt] = Field(default=None)
    text_generation: typing.Optional[TextGenerationUserPrompt] = Field(default=None)
    mistral: typing.Optional[MistralUserPrompt] = Field(default=None)
    azure_mistral: typing.Optional[AzureMistralUserPrompt] = Field(default=None)
    claude3: typing.Optional[Claude3UserPrompt] = Field(default=None)


class SemanticConfig(BaseModel):
    """
    Keep this in sync with learning_models.encoder.SemanticConfig
    """

    # Similarity function to use for semantic similarity
    similarity: SimilarityFunction = Field(default=SimilarityFunction.DOT)
    # Number of dimensions of the embeddings
    size: int = Field(default=0)
    # Similarity threshold to use at search time, results with a similarity below this threshold will be discarded
    threshold: float = Field(default=0.0)
    # Maximum number of tokens in a sentence that the model can handle
    max_tokens: int = Field(default=0)
    # Subdivisions of the matryoshka embeddings that can be used (if the model supports it), in descending order
    matryoshka_dims: typing.List[int] = Field(default_factory=list)
    # Whether the model is external
    external: bool = Field(default=False)


class LearningConfiguration(BaseModel):
    """
    Hey, developer! Keep this in sync with corresponding pydantic model in learning_config.models
    """

    semantic_model: str = Field(default="")
    anonymization_model: str = Field(default="")
    ner_model: str = Field(default="")
    visual_labeling: str = Field(default="")
    relation_model: str = Field(default="")
    summary: str = Field(default="")
    summary_model: str = Field(default="")
    summary_provider: str = Field(default="")
    summary_prompt_id: str = Field(default="")
    summary_prompt: SummaryPrompt = Field()
    user_keys: UserLearningKeys = Field()
    user_prompts: UserPrompts = Field()
    prefer_markdown_generative_response: bool = Field(default=False)
    semantic_models: typing.List[str] = Field(default_factory=list)
    semantic_model_configs: typing.Dict[str, SemanticConfig] = Field(default_factory=dict)
    default_semantic_model: str = Field(default="")
    generative_model: str = Field(default="")
    generative_provider: str = Field(default="")
    generative_prompt_id: str = Field(default="")


class AddKBConfigRequest(BaseModel):
    kbid: str = Field(default="")
    account: str = Field(default="")
    config: LearningConfiguration = Field()


class UpdateKBConfigRequest(BaseModel):
    kbid: str = Field(default="")
    config: LearningConfiguration = Field()


class UpdateKBConfigResponse(BaseModel):
    status: ResponseStatus = Field(default=ResponseStatus.OK)


class DeleteKBConfigRequest(BaseModel):
    kbid: str = Field(default="")


class DeleteKBConfigResponse(BaseModel):
    status: ResponseStatus = Field(default=ResponseStatus.OK)


class GetKBConfigRequest(BaseModel):
    kbid: str = Field(default="")


class GetExternalKBConfigRequest(BaseModel):
    account: str = Field(default="")
    kbid: str = Field(default="")


class GetKBConfigResponse(BaseModel):
    status: ResponseStatus = Field(default=ResponseStatus.OK)
    config: LearningConfiguration = Field()
    semantic_vector_similarity: str = Field(default="")
    semantic_vector_size: int = Field(default=0)
    semantic_threshold: float = Field(default=0.0)
    errors: typing.List[str] = Field(default_factory=list)
    semantic_matryoshka_dimensions: typing.List[int] = Field(default_factory=list)


class AddModelRequest(BaseModel):
    model_type: ModelType = Field(default=ModelType.GENERATIVE)
    trained_date: datetime = Field(default_factory=datetime.now)
    location: str = Field(default="")
    log: str = Field(default="")
    loss: float = Field(default=0.0)
    accuracy: float = Field(default=0.0)
    account: str = Field(default="")
    trained_kbid: str = Field(default="")
    model_uuid: str = Field(default="")
    trained_dataset: str = Field(default="")
    title: str = Field(default="")


class AddModelResponse(BaseModel):
    uuid: str = Field(default="")


class DeleteModelRequest(BaseModel):
    model_id: str = Field(default="")
    account_id: str = Field(default="")


class DeleteModelResponse(BaseModel):
    pass


class GetModelsRequest(BaseModel):
    kbid: str = Field(default="")


class GetExternalModelsRequest(BaseModel):
    kbid: str = Field(default="")
    account: str = Field(default="")


class Model(BaseModel):
    model_type: ModelType = Field(default=ModelType.GENERATIVE)
    trained_date: datetime = Field(default_factory=datetime.now)
    model_id: str = Field(default="")
    account: str = Field(default="")
    trained_kbid: str = Field(default="")
    trained_dataset: str = Field(default="")
    title: str = Field(default="")
    provider: str = Field(default="")
    prompt_id: str = Field(default="")
    kbids: typing.List[str] = Field(default_factory=list)


class GetModelsResponse(BaseModel):
    models: typing.List[Model] = Field(default_factory=list)


class SetAvailableModelsRequest(BaseModel):
    kbid: str = Field(default="")
    models: typing.List[str] = Field(default_factory=list)


class SetAvailableModelsResponse(BaseModel):
    pass


class GetModelRequest(BaseModel):
    model_id: str = Field(default="")


class GetAccountModelsRequest(BaseModel):
    account: str = Field(default="")
    client_id: str = Field(default="")


class GetModelResponse(BaseModel):
    model_type: ModelType = Field(default=ModelType.GENERATIVE)
    trained_date: datetime = Field(default_factory=datetime.now)
    location: str = Field(default="")
    log: str = Field(default="")
    loss: float = Field(default=0.0)
    accuracy: float = Field(default=0.0)
    account: str = Field(default="")
    trained_kbid: str = Field(default="")
    model_id: str = Field(default="")
    kbids: typing.List[str] = Field(default_factory=list)


class DeleteTrainedModelsOfAccountRequest(BaseModel):
    account: str = Field(default="")


class DeleteTrainedModelsOfAccountResponse(BaseModel):
    class Status(IntEnum):
        OK = 0
        ERROR = 1

    status: "DeleteTrainedModelsOfAccountResponse.Status" = Field(default=Status.OK)


###
#    learning_protos/data_augmentation_p2p.py
###


class ApplyTo(IntEnum):
    TEXT_BLOCK = 0
    FIELD = 1


class KBConfiguration(BaseModel):
    account: str = Field(default="")
    kbid: str = Field(default="")
    onprem: bool = Field(default=False)


class Trigger(BaseModel):
    url: str = Field(default="")
    headers: typing.Dict[str, str] = Field(default_factory=dict)
    params: typing.Dict[str, str] = Field(default_factory=dict)


class EntityDefinition(BaseModel):
    label: str = Field(description="Entity type")
    description: typing.Optional[str] = Field(default="", description="Description of the entity type")


class EntityExample(BaseModel):
    name: str = Field(description="Name associated with the entity")
    label: str = Field(description="Type of entity")


class RelationExample(BaseModel):
    source: str = Field(description="Entity name from which the relation starts")
    target: str = Field(description="Entity name to which the relation ends")
    label: str = Field(description="Type of relation")


class GraphExtractionExample(BaseModel):
    entities: typing.List[EntityExample] = Field(
        description="Examples of entities extracted from the example text"
    )
    relations: typing.List[RelationExample] = Field(
        description="Examples of relations extracted from the example text, all entities must be included in the entities list"
    )
    text: str = Field(description="Example text where entities and relations were extracted")


class GraphOperation(BaseModel):
    entity_defs: typing.List[EntityDefinition] = Field(
        default_factory=list,
        description="Types of entities that will be considered for extraction",
    )
    examples: typing.List[GraphExtractionExample] = Field(
        default_factory=list,
        description="Examples to be sent to the model as few-shot learning",
    )
    ident: str = Field(title="ID of Operation, Will be used to tag the extracted entities and relationships")
    triggers: typing.List[Trigger] = Field(
        default_factory=list, title="Triggers to execute when the operation is executed"
    )


class Label(BaseModel):
    label: str = Field(title="Label Name")
    description: str = Field(default="", title="Description of the Label")
    examples: typing.List[str] = Field(default_factory=list)


class LabelOperation(BaseModel):
    labels: typing.List[Label] = Field(default_factory=list)
    ident: str = Field(title="ID of Operation, Will be used to name the generated Label Set")
    description: str = Field(
        default="",
        title="Description of the Operation, will be used to describe to the model the task at hand",
    )
    multiple: bool = Field(default=False)
    triggers: typing.List[Trigger] = Field(
        default_factory=list, title="Triggers to execute when the operation is executed"
    )


class GuardOperation(BaseModel):
    enabled: bool = Field(default=False)
    triggers: typing.List[Trigger] = Field(
        default_factory=list, title="Triggers to execute when the operation is executed"
    )


class AskOperation(BaseModel):
    model_config = ConfigDict(serialize_by_alias=True)

    question: str = Field(title="Question")
    destination: str = Field(title="Destination")
    generate_json: bool = Field(default=False, alias="json")
    triggers: typing.List[Trigger] = Field(
        default_factory=list, title="Triggers to execute when the operation is executed"
    )
    user_prompt: str = Field(
        default="",
        title="Custom User Prompt",
        description="Prompt to use when executing the agent over a field, it must include the {context} placeholder and optionally the {question} placeholder if you wish. If not given, the default prompt will be used. Only supported when `json` is not set",
    )


class QAOperation(BaseModel):
    question_generator_prompt: str = Field(default="")
    system_question_generator_prompt: str = Field(default="")
    summary_prompt: str = Field(default="")
    generate_answers_prompt: str = Field(default="")
    triggers: typing.List[Trigger] = Field(
        default_factory=list, title="Triggers to execute when the operation is executed"
    )


class ExtractOperation(BaseModel):
    class Model(IntEnum):
        TABLES = 0

    model: "ExtractOperation.Model" = Field(default=Model.TABLES)
    triggers: typing.List[Trigger] = Field(
        default_factory=list, title="Triggers to execute when the operation is executed"
    )


class Operation(BaseModel):
    graph: typing.Optional[GraphOperation] = Field(default=None, title="Graph Config")
    label: typing.Optional[LabelOperation] = Field(default=None, title="Label Config")
    ask: typing.Optional[AskOperation] = Field(default=None, title="Ask Config")
    qa: typing.Optional[QAOperation] = Field(default=None, title="QA Config")
    extract: typing.Optional[ExtractOperation] = Field(default=None, title="Extract Config")
    prompt_guard: typing.Optional[GuardOperation] = Field(default=None, title="Prompt Guard Config")
    llama_guard: typing.Optional[GuardOperation] = Field(default=None, title="Llama Guard Config")


class FilterLogicalOperator(IntEnum):
    AND = 0
    OR = 1


class Filter(BaseModel):
    contains: typing.List[str] = Field(
        default_factory=list,
        description="Text that must be contained in the field in order to apply the data augmentation, if multiple values are provided, at least one must be present",
    )
    resource_type: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will only be applied to resources of the specified types",
    )
    field_types: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will only be applied to fields of the specified types",
    )
    not_field_types: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will not be applied to fields of the specified types",
    )
    rids: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will only be applied resources matching the specified resource ids",
    )
    fields: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will only be applied to fields matching the specified field ids",
    )
    splits: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will only be applied to fields matching the specified field splits",
    )
    labels: typing.List[str] = Field(
        default_factory=list,
        description="Data Augmentation will only be applied to fields matching the specified labels",
    )
    apply_to_agent_generated_fields: bool = Field(
        default=False,
        description="If enabled, the data augmentation will also be applied to fields generated by other data augmentation agents. This functionality is only supported for agents that do not generate fields.",
    )
    contains_operator: FilterLogicalOperator = Field(
        default=FilterLogicalOperator.AND, description="Way of combining the values in the contains field"
    )
    labels_operator: FilterLogicalOperator = Field(
        default=FilterLogicalOperator.AND, description="Way of combining the values in the labels field"
    )


class LLMConfig(BaseModel):
    model: str = Field(default="")
    provider: str = Field(default="")
    keys: typing.Optional[UserLearningKeys] = Field(default=None)
    prompts: typing.Optional[UserPrompts] = Field(default=None)


class DataAugmentation(BaseModel):
    # If you modify this proto remember to update nuclia-models repo accordingly
    name: str = Field(default="")
    on: ApplyTo = Field(
        default=ApplyTo.TEXT_BLOCK,
        title="ApplyTo",
        description="Defines if the task should be applied to paragraphs (0) or whole fields (1)",
    )
    filter: typing.Optional[Filter] = Field(default=None, description="Filter to apply the data augmentation")
    operations: typing.List[Operation] = Field(default_factory=list)
    llm: LLMConfig = Field()


class DataAugmentations(BaseModel):
    class Status(IntEnum):
        FOUND = 0
        NOT_FOUND = 1

    data_augmentations: typing.List[DataAugmentation] = Field(default_factory=list)
    status: "DataAugmentations.Status" = Field(default=Status.FOUND)
